@@ -4,6 +4,7 @@
 require("chrome-extension-async");
 const sha1 = require("sha1");
 const idb = require("idb");
+const BrowserpassURL = require("@browserpass/url");
 const helpers = require("./helpers");
 
 // native application id
@@ -23,11 +24,22 @@ var defaultSettings = {
     gpgPath: null,
     stores: {},
     foreignFills: {},
-    username: null
+    username: null,
+    theme: "dark"
 };
 
 var authListeners = {};
 
+var badgeCache = {
+    files: null,
+    settings: null,
+    expires: Date.now(),
+    isRefreshing: false
+};
+
+// the last text copied to the clipboard is stored here in order to be cleared after 60 seconds
+let lastCopiedText = null;
+
 chrome.browserAction.setBadgeBackgroundColor({
     color: "#666"
 });
@@ -42,10 +54,8 @@ chrome.tabs.onUpdated.addListener((tabId, info) => {
         }
     }
 
-    // refresh badge counter when url in a tab changes
-    if (info.url) {
-        updateMatchingPasswordsCount(tabId);
-    }
+    // redraw badge counter
+    updateMatchingPasswordsCount(tabId);
 });
 
 // handle incoming messages
@@ -80,6 +90,16 @@ chrome.commands.onCommand.addListener(async command => {
     }
 });
 
+// handle fired alarms
+chrome.alarms.onAlarm.addListener(alarm => {
+    if (alarm.name === "clearClipboard") {
+        if (readFromClipboard() === lastCopiedText) {
+            copyToClipboard("", false);
+        }
+        lastCopiedText = null;
+    }
+});
+
 chrome.runtime.onInstalled.addListener(onExtensionInstalled);
 
 //----------------------------------- Function definitions ----------------------------------//
@@ -89,52 +109,69 @@ chrome.runtime.onInstalled.addListener(onExtensionInstalled);
  *
  * @since 3.0.0
  *
- * @param int tabId Tab id
+ * @param int  tabId Tab id
+ * @param bool forceRefresh force invalidate cache
  * @return void
  */
-async function updateMatchingPasswordsCount(tabId) {
+async function updateMatchingPasswordsCount(tabId, forceRefresh = false) {
+    if (badgeCache.isRefreshing) {
+        return;
+    }
+
     try {
-        const settings = await getFullSettings();
-        var response = await hostAction(settings, "list");
-        if (response.status != "ok") {
-            throw new Error(JSON.stringify(response));
+        if (forceRefresh || Date.now() > badgeCache.expires) {
+            badgeCache.isRefreshing = true;
+
+            let settings = await getFullSettings();
+            let response = await hostAction(settings, "list");
+            if (response.status != "ok") {
+                throw new Error(JSON.stringify(response));
+            }
+
+            const CACHE_TTL_MS = 60 * 1000;
+            badgeCache = {
+                files: response.data.files,
+                settings: settings,
+                expires: Date.now() + CACHE_TTL_MS,
+                isRefreshing: false
+            };
         }
 
-        // Get tab info
         try {
             const tab = await chrome.tabs.get(tabId);
-            settings.host = new URL(tab.url).hostname;
+            badgeCache.settings.origin = new URL(tab.url).origin;
         } catch (e) {
             throw new Error(`Unable to determine domain of the tab with id ${tabId}`);
         }
 
-        const logins = helpers.prepareLogins(response.data.files, settings);
+        // Compule badge counter
+        const files = helpers.ignoreFiles(badgeCache.files, badgeCache.settings);
+        const logins = helpers.prepareLogins(files, badgeCache.settings);
         const matchedPasswordsCount = logins.reduce(
-            (acc, login) => acc + (login.recent.count || login.inCurrentDomain ? 1 : 0),
+            (acc, login) => acc + (login.recent.count || login.inCurrentHost ? 1 : 0),
             0
         );
 
-        if (matchedPasswordsCount) {
-            // Set badge for the current tab
-            chrome.browserAction.setBadgeText({
-                text: "" + matchedPasswordsCount,
-                tabId: tabId
-            });
-        }
+        // Set badge for the current tab
+        chrome.browserAction.setBadgeText({
+            text: "" + (matchedPasswordsCount || ""),
+            tabId: tabId
+        });
     } catch (e) {
         console.log(e);
     }
 }
 
 /**
- * Copy text to clipboard
+ * Copy text to clipboard and optionally clear it from the clipboard after one minute
  *
- * @since 3.0.0
+ * @since 3.2.0
  *
  * @param string text Text to copy
+ * @param boolean clear Whether to clear the clipboard after one minute
  * @return void
  */
-function copyToClipboard(text) {
+function copyToClipboard(text, clear = true) {
     document.addEventListener(
         "copy",
         function(e) {
@@ -144,6 +181,31 @@ function copyToClipboard(text) {
         { once: true }
     );
     document.execCommand("copy");
+
+    if (clear) {
+        lastCopiedText = text;
+        chrome.alarms.create("clearClipboard", { delayInMinutes: 1 });
+    }
+}
+
+/**
+ * Read plain text from clipboard
+ *
+ * @since 3.2.0
+ *
+ * @return string The current plaintext content of the clipboard
+ */
+function readFromClipboard() {
+    const ta = document.createElement("textarea");
+    // these lines are carefully crafted to make paste work in both Chrome and Firefox
+    ta.contentEditable = true;
+    ta.textContent = "";
+    document.body.appendChild(ta);
+    ta.select();
+    document.execCommand("paste");
+    const content = ta.value;
+    document.body.removeChild(ta);
+    return content;
 }
 
 /**
@@ -168,14 +230,15 @@ async function saveRecent(settings, login, remove = false) {
         login.recent.count++;
     }
     login.recent.when = Date.now();
-    settings.recent[sha1(settings.host + sha1(login.store.id + sha1(login.login)))] = login.recent;
+    settings.recent[sha1(settings.origin + sha1(login.store.id + sha1(login.login)))] =
+        login.recent;
 
     // save to local storage
     localStorage.setItem("recent", JSON.stringify(settings.recent));
 
     // a new entry was added to the popup matching list, need to refresh the count
-    if (!login.inCurrentDomain && login.recent.count === 1) {
-        updateMatchingPasswordsCount(settings.tab.id);
+    if (!login.inCurrentHost && login.recent.count === 1) {
+        updateMatchingPasswordsCount(settings.tab.id, true);
     }
 
     // save to usage log
@@ -186,7 +249,7 @@ async function saveRecent(settings, login, remove = false) {
                 db.createObjectStore("log", { keyPath: "time" });
             }
         });
-        await db.add("log", { time: Date.now(), host: settings.host, login: login.login });
+        await db.add("log", { time: Date.now(), host: settings.origin, login: login.login });
     } catch {
         // ignore any errors and proceed without saving a log entry to Indexed DB
     }
@@ -206,7 +269,7 @@ async function dispatchFill(settings, request, allFrames, allowForeign, allowNoS
     request = Object.assign(deepCopy(request), {
         allowForeign: allowForeign,
         allowNoSecret: allowNoSecret,
-        foreignFills: settings.foreignFills[settings.host] || {}
+        foreignFills: settings.foreignFills[settings.origin] || {}
     });
 
     let perFrameResults = await chrome.tabs.executeScript(settings.tab.id, {
@@ -224,10 +287,10 @@ async function dispatchFill(settings, request, allFrames, allowForeign, allowNoS
     let foreignFillsChanged = false;
     for (let frame of perFrameResults) {
         if (typeof frame.foreignFill !== "undefined") {
-            if (typeof settings.foreignFills[settings.host] === "undefined") {
-                settings.foreignFills[settings.host] = {};
+            if (typeof settings.foreignFills[settings.origin] === "undefined") {
+                settings.foreignFills[settings.origin] = {};
             }
-            settings.foreignFills[settings.host][frame.foreignOrigin] = frame.foreignFill;
+            settings.foreignFills[settings.origin][frame.foreignOrigin] = frame.foreignFill;
             foreignFillsChanged = true;
         }
     }
@@ -250,7 +313,7 @@ async function dispatchFill(settings, request, allFrames, allowForeign, allowNoS
 async function dispatchFocusOrSubmit(settings, request, allFrames, allowForeign) {
     request = Object.assign(deepCopy(request), {
         allowForeign: allowForeign,
-        foreignFills: settings.foreignFills[settings.host] || {}
+        foreignFills: settings.foreignFills[settings.origin] || {}
     });
 
     let perFrameResults = await chrome.tabs.executeScript(settings.tab.id, {
@@ -357,7 +420,7 @@ async function fillFields(settings, login, fields) {
         // try again using all available frames if we couldn't fill an "important" field
         if (
             !filledFields.includes(importantFieldToFill) &&
-            settings.foreignFills[settings.host] !== false
+            settings.foreignFills[settings.origin] !== false
         ) {
             allowForeign = true;
             filledFields = filledFields.concat(
@@ -395,7 +458,7 @@ async function fillFields(settings, login, fields) {
             }
 
             // try again using all available frames
-            if (!filledFields.length && settings.foreignFills[settings.host] !== false) {
+            if (!filledFields.length && settings.foreignFills[settings.origin] !== false) {
                 allowForeign = true;
                 filledFields = filledFields.concat(
                     await dispatchFill(
@@ -521,7 +584,9 @@ async function getFullSettings() {
     // Fill current tab info
     try {
         settings.tab = (await chrome.tabs.query({ active: true, currentWindow: true }))[0];
-        settings.host = new URL(settings.tab.url).hostname;
+        let originInfo = new BrowserpassURL(settings.tab.url);
+        settings.host = originInfo.host; // TODO remove this after OTP extension is migrated
+        settings.origin = originInfo.origin;
     } catch (e) {}
 
     return settings;
@@ -584,7 +649,7 @@ function handleModalAuth(requestDetails) {
     if (launchHost !== requestDetails.challenger.host) {
         var message =
             "You are about to send login credentials to a domain that is different than " +
-            "the one you lauched from the browserpass extension. Do you wish to proceed?\n\n" +
+            "the one you launched from the browserpass extension. Do you wish to proceed?\n\n" +
             "Realm: " +
             requestDetails.realm +
             "\n" +
@@ -675,7 +740,8 @@ async function handleMessage(settings, message, sendResponse) {
                 if (response.status != "ok") {
                     throw new Error(JSON.stringify(response)); // TODO handle host error
                 }
-                sendResponse({ status: "ok", files: response.data.files });
+                let files = helpers.ignoreFiles(response.data.files, settings);
+                sendResponse({ status: "ok", files });
             } catch (e) {
                 sendResponse({
                     status: "error",
@@ -711,7 +777,7 @@ async function handleMessage(settings, message, sendResponse) {
         case "launch":
         case "launchInNewTab":
             try {
-                var url = message.login.fields.url || message.login.domain;
+                var url = message.login.fields.url || message.login.host;
                 if (!url) {
                     throw new Error("No URL is defined for this entry");
                 }
@@ -1023,6 +1089,7 @@ function triggerOTPExtension(settings, action, otp) {
                 otp: otp,
                 settings: {
                     host: settings.host,
+                    origin: settings.origin,
                     tab: settings.tab
                 }
             })
@@ -1069,6 +1136,7 @@ function onExtensionInstalled(details) {
         }
     } else if (details.reason === "update") {
         var changelog = {
+            3002000: "New permissions added to clear copied credentials after 60 seconds.",
             3000000:
                 "New major update is out, please update the native host app to v3.\n" +
                 "Instructions here: https://github.com/browserpass/browserpass-native"
